#include <stdio.h>
#include <iostream>

#include "GPBPatch.hpp"
#include "SourceOperator.hpp"
#include <Event.pb.h>

SourceOperator::SourceOperator()
{
    logger = log4cxx::Logger::getLogger("SourceOperator");

    sliceId = 0;
    channels = new std::map<int, JBossNettyAdapter*>();
    channelsLock = new pthread_mutex_t;
    pthread_mutex_init(channelsLock, NULL);
}

void SourceOperator::init(int sliceId, int sliceUId, int bucketCount, char* param, int size)
{
    this->sliceId = sliceId;
}

void SourceOperator::onConnectAdapter(int portId, int sourceId, Collector* collector)
{
    LOG4CXX_INFO(logger, "Connected to external port/source # " << portId << "/" << sourceId);

    pthread_mutex_lock(channelsLock);
    if (channels->find(sourceId) == channels->end()) (*channels)[sourceId] = new JBossNettyAdapter(this);
    pthread_mutex_unlock(channelsLock);

    char* buf = new char[1];
    collector->readAdapter(buf, 1, sourceId);
}

void SourceOperator::onReadAdapter(void* buf, int size, int sourceId, Collector* collector)
{
    pthread_mutex_lock(channelsLock);
    int newSize = (*channels)[sourceId]->onRead(buf, size, sliceId, collector);
    pthread_mutex_unlock(channelsLock);

    delete [](char*)buf;
    buf = new char[newSize];
    collector->readAdapter(buf, newSize, sourceId);
}

void SourceOperator::onMessage(void* buf, int size, int sliceId, Collector* collector)
{
    Financial::Event* event = new Financial::Event();
    event->ParseFromArray(buf, size);

    void* eventOut = collector->allocEvent(sliceId, event->timestamp(), size);
    memcpy(eventOut, buf, size);
    collector->commit();

    LOG4CXX_INFO(logger, "Received event from external port/source with ts: " <<
        event->timestamp() << " and tick: " << event->tick());

    delete event;
}

bool SourceOperator::generate(Collector* collector)
{
    collector->bindAdapter((char*)"localhost", 20001);
    return false;
}

void SourceOperator::process(int routingKey, void* eventIn, int size, void* state, Collector* collector)
{
    Financial::Event* event = new Financial::Event();
    event->ParseFromArray(eventIn, size);

    // simple filter
    if (event->tick() > 1000)
    {
        LOG4CXX_INFO(logger, "Discarding event with ts: " <<
            event->timestamp() << " and tick: " << event->tick());
        delete event;
        return;
    }

    LOG4CXX_INFO(logger, "Forwarding event with ts: " <<
        event->timestamp() << " and tick: " << event->tick());

    // if not filter out, simply forward
    void *eventOut = collector->allocEvent(event->stockid(), event->timestamp(), size);
    memcpy(eventOut, eventIn, size);
    collector->commit();

    delete event;
}

SourceOperator::~SourceOperator()
{
    for (std::map<int, JBossNettyAdapter*>::iterator iter = channels->begin(); iter != channels->end(); ++iter) delete iter->second;
    delete channels;
    delete channelsLock;

    GPBPatch::ShutdownProtobufLibrary();
}

extern "C"
{
    Operator *operatorInstance()
    {
        return new SourceOperator();
    }
}
