#include <stdio.h>
#include <string.h>
#include <iostream>

#include "GPBPatch.hpp"
#include "WorkerOperator.hpp"
#include <Event.pb.h>

WorkerOperator::WorkerOperator()
{
    logger = log4cxx::Logger::getLogger("WorkerOperator");
}

void* WorkerOperator::stateInit()
{
    State* state = new State();
    state->count = 0;
    for (int i=0; i<WINDOWSIZE; i++) state->ticks[i] = 0;
    return state;
}

int WorkerOperator::getStateSize(void* state)
{
    return sizeof(State);
}

void WorkerOperator::freeState(void* state)
{
    delete (State*)state;
}

void WorkerOperator::serializeState(void* state, std::list<Buffer*>* buffers)
{
    Buffer* buffer = new Buffer(state, sizeof(State));
    buffers->push_back(buffer);
}

void* WorkerOperator::deserializeState(std::list<Buffer*>* buffers)
{
    void* state = stateInit();
    memcpy(state, buffers->front()->getData(), buffers->front()->getSize());
    return state;
}

void WorkerOperator::stateAccess(int routingKey, void* eventIn, int size, StateAccess* stateAccess)
{
    if (routingKey != -1) stateAccess->setKey(routingKey % 10);
    else stateAccess->setKey(0);
}

void WorkerOperator::process(int routingKey, void* eventIn, int size, void* state, Collector* collector)
{
    Financial::Event* event = new Financial::Event();
    event->ParseFromArray(eventIn, size);

    State* myState = (State*)state;

    myState->count++;
    for (int i=1; i<WINDOWSIZE; i++) myState->ticks[i-1] = myState->ticks[i];
    myState->ticks[WINDOWSIZE-1] = event->tick();

    float movingAvg = 0;
    if (myState->count % 2 == 0 && myState->count > 10)
    {
        int sum = 0;
        for (int i=0; i<WINDOWSIZE; i++) sum += myState->ticks[i];

        movingAvg = (float)sum/(float)WINDOWSIZE;

        LOG4CXX_INFO(logger, "movingAvg: " << movingAvg);
    }

    delete event;
}

WorkerOperator::~WorkerOperator()
{
    GPBPatch::ShutdownProtobufLibrary();
}

extern "C"
{
    Operator *operatorInstance()
    {
        return new WorkerOperator();
    }
}
